package protocolP2P;
import exception.ProtocolError;
import exception.VersionError;
import protocolP2P.Payload;
import protocolP2P.RequestResponseCode;
import java.util.ArrayList;
import java.lang.Byte;

public class ProtocolP2PDatagram {
	private final static byte PROTOCOL_VERSION = 0x11;
	private final static int VERSION_POSITON = 0;
	private byte version;
	private Payload payload;

	public ProtocolP2PDatagram(Payload payload) {
		version = PROTOCOL_VERSION;
		this.payload = payload;
	}

	public ProtocolP2PDatagram(byte[] datagram) throws ProtocolError, VersionError {
		// unwrap version
		version = datagram[VERSION_POSITON];
		checkProtocolVersion(); // this can throw VersionError
		RequestResponseCode r = RequestResponseCode.fromCode(datagram[RequestResponseCode.RRCODE_POSITION]); // this can throw ProtocolError
		switch (r) {
			case RequestResponseCode.LIST_RESPONSE:
				payload = (Payload) new FileList(datagram);
				break;
			case RequestResponseCode.LOAD_RESPONSE:
				payload = (Payload) new FilePart(datagram);
				break;
			default:
				payload = new Payload(datagram);
				break;
			}
	}

	public byte[] toDatagram() {
		byte[] datagram = payload.toDatagram();
		datagram[VERSION_POSITON] = version;
		return datagram;
	}

	public Payload getPayload() {
		return payload;
	}

	private void checkProtocolVersion() throws VersionError {
		if (PROTOCOL_VERSION != version) {
			throw new VersionError();
		}
	}
}

